/***************************************************************************/
/*                                                                         */
/*  File: Cube.cpp                                                         */
/*  Date: 24-10-07                                                         */
/*                                                                         */
/***************************************************************************/
/*
	See .h file
*/
/***************************************************************************/


#include <windows.h>

#include <stdio.h>					// sprintf(..)
#include <d3dx9.h>					// Direct X Header Files
#pragma comment(lib, "d3d9.lib")	// Direct X Library Files	
#pragma comment(lib, "d3dx9.lib")

#include "Cube.h"


void Cube::Render(IDirect3DDevice9* pDevice)
{
	struct TLVERTEX
	{
		float x,y,z;
		unsigned int colour;
		enum{ FVF_TLVERTEX = D3DFVF_XYZ | D3DFVF_DIFFUSE };
	};

	pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);
	//pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	pDevice->SetTextureStageState(0,D3DTSS_COLOROP,	D3DTOP_SELECTARG1);
	pDevice->SetTextureStageState(0,D3DTSS_COLORARG1,	D3DTA_DIFFUSE);
	float w = m_e.x;
	float h = m_e.y;
	float d = m_e.z;
	DWORD col = 0xffff00ff;

	// Add some random colouring
	const unsigned int rand = 0xffffffff;

	TLVERTEX Vertex[36] = 
	{
		//   x     y      z       colour
		// Front
		{   -w,   -h,	-d,    0xffff0000 & rand},
		{   +w,   +h,	-d,    0xffff0000 & rand},
		{   -w,   +h,	-d,    0xffff0000 & rand},

		{   -w,   -h,	-d,    0xffff0000 & rand},
		{   +w,   -h,	-d,    0xffff0000 & rand},
		{   +w,   +h,	-d,    0xffff0000 & rand},

		// Back
		{   -w,   -h,	+d,    0xff00ff00 & rand},
		{   -w,   +h,	+d,    0xff00ff00 & rand},
		{   +w,   +h,	+d,    0xff00ff00 & rand},

		{   -w,   -h,	+d,    0xff00ff00 & rand},
		{   +w,   +h,	+d,    0xff00ff00 & rand},
		{   +w,   -h,	+d,    0xff00ff00 & rand},

		// Top
		{   -w,   +h,	-d,    0xff0000ff & rand},
		{   +w,   +h,	+d,    0xff0000ff & rand},
		{   -w,   +h,	+d,    0xff0000ff & rand},

		{   -w,   +h,	-d,    0xff0000ff & rand},
		{   +w,   +h,	-d,    0xff0000ff & rand},
		{   +w,   +h,	+d,    0xff0000ff & rand},

		// Bottom
		{   -w,   -h,	-d,    0xffffff0f & rand},
		{   -w,   -h,	+d,    0xffffff0f & rand},
		{   +w,   -h,	+d,    0xffffff0f & rand},

		{   -w,   -h,	-d,    0xffffff00 & rand},
		{   +w,   -h,	+d,    0xffffff00 & rand},
		{   +w,   -h,	-d,    0xffffff00 & rand},

		// Left
		{   -w,   -h,	-d,    0xffffffff & rand},
		{   -w,   +h,	-d,    0xffffffff & rand},
		{   -w,   +h,	+d,    0xffffffff & rand},

		{   -w,   -h,	-d,    0xffffffff & rand},
		{   -w,   +h,	+d,    0xffffffff & rand},
		{   -w,   -h,	+d,    0xffffffff & rand},

		// Right
		{   +w,   -h,	-d,    0xff000000 & rand},
		{   +w,   +h,	+d,    0xff000000 & rand},
		{   +w,   +h,	-d,    0xff000000 & rand},

		{   +w,   -h,	-d,    0xff000000 & rand},
		{   +w,   -h,	+d,    0xff000000 & rand},
		{   +w,   +h,	+d,    0xff000000 & rand},
	};
	pDevice->SetTexture( 0, NULL);


	#if(DIRECT3D_VERSION >= 0x0900)
	pDevice->SetFVF( TLVERTEX::FVF_TLVERTEX );
	#else
	pDevice->SetVertexShader( TLVERTEX::FVF_TLVERTEX );
	#endif // DIRECT3D_VERSION

	//D3DXMATRIX matMove, matScale, matRot, matWorld;
	//D3DXMatrixTranslation(&matMove, pos.x, pos.y, pos.z);
	//D3DXMatrixScaling(&matScale, dim.x, dim.y, dim.z);
	//D3DXMatrixRotationYawPitchRoll(&matRot, rot.y, rot.x, rot.z);

	pDevice->SetTransform(D3DTS_WORLD, &m_matWorld);

	HRESULT hr = 
	pDevice->DrawPrimitiveUP( D3DPT_TRIANGLELIST, 12, Vertex, sizeof( TLVERTEX ) );

	UpdateMatrix();
}// End Render(..)


void Cube::Update(float dt)
{
	if (m_mass > 1000.0f)
	{
		return;
	}

	if (D3DXVec3Length(&m_linVelocity) < 0.0001f)  m_linVelocity = D3DXVECTOR3(0,0,0);
	if (D3DXVec3Length(&m_angVelocity) < 0.0001f)  m_angVelocity = D3DXVECTOR3(0,0,0);



	AddForce( m_c, D3DXVECTOR3(0, -9.8f * m_mass, 0) );


	m_linAcceleration = m_forces / m_mass;  // F = ma

	m_angAcceleration = m_torques * m_invInertia;


	// Update position
	m_c += m_linVelocity * dt  +  m_linAcceleration*(dt*dt*0.5f);


	// Update Angular
	m_angVelocity += (m_angAcceleration * dt);

	D3DXQUATERNION Qvel = 0.5f * m_rot * D3DXQUATERNION(m_angVelocity.x, m_angVelocity.y, m_angVelocity.z, 0);

	D3DXQUATERNION Qaccel = 0.5f * m_rot * D3DXQUATERNION(m_angAcceleration.x, m_angAcceleration.y, m_angAcceleration.z, 0);

	m_rot += Qvel * dt;


	D3DXQuaternionNormalize(&m_rot, &m_rot);


	// Update Linear
	m_linVelocity += m_linAcceleration*dt;

	// Bit Damping
	const float damping = 0.98f;

	if (D3DXVec3Length(&m_linVelocity) > 5.0f)
	{
		m_linVelocity *= 0.8f;
	}

	m_linVelocity *= damping;
	m_angVelocity *= damping;


	// Reset Forces
	m_linAcceleration	= D3DXVECTOR3(0,0,0);
	m_angAcceleration	= D3DXVECTOR3(0,0,0);
	m_forces			= D3DXVECTOR3(0,0,0);
	m_torques			= D3DXVECTOR3(0,0,0);


	UpdateMatrix();
}





















