/***************************************************************************/
/*                                                                         */
/*  File: Cube.h                                                           */
/*  Date: 24-10-07                                                         */
/*                                                                         */
/***************************************************************************/
/*
*/
/***************************************************************************/
#ifndef CUBE_H
#define CUBE_H


#include <d3dx9.h>				// Direct X Header Files

								// Direct X Library Files
#pragma comment(lib, "d3d9.lib")	
#pragma comment(lib, "d3dx9.lib")

inline D3DXVECTOR3 Cross(const D3DXVECTOR3& vA, const D3DXVECTOR3& vB)
{
	D3DXVECTOR3 temp;
	D3DXVec3Cross(&temp, &vA, &vB);
	return temp;
}

inline float Dot(const D3DXVECTOR3& vA, const D3DXVECTOR3& vB)
{
	return D3DXVec3Dot(&vA, &vB);
}

inline D3DXVECTOR3 operator* (D3DXVECTOR3& vA, D3DXMATRIX& m)
{
	D3DXVECTOR3 temp;
	D3DXVec3TransformCoord(&temp, &vA, &m);
	return temp;
}

inline D3DXMATRIX Inverse(D3DXMATRIX& m)
{
	D3DXMATRIX temp;
	D3DXMatrixInverse(&temp, NULL, &m);
	return temp;
}

inline D3DXVECTOR3 Normalize(D3DXVECTOR3& vA)
{
	D3DXVECTOR3 temp;
	D3DXVec3Normalize(&temp, &vA);
	return temp;
}


inline float Clamp(float v, float min, float max)
{
	float res = v;
	res = v>max ? max : v;
	res = v<min ? min : v;
	return res;
};


/*
enum eCollision
{
	COLLISION_NONE,
	COLLISION_HIT,
	COLLISION_PENETRATING
};
*/



class Cube
{
public:
	D3DXVECTOR3 m_c;			// Center Of Cube
	D3DXVECTOR3 m_u[3];			// Local x-y-z Axes
	D3DXVECTOR3 m_e;			// Positive halfwidths along each axis

	D3DXMATRIX  m_matWorld;
	D3DXQUATERNION m_rot;


	float		m_mass;
	D3DXMATRIX	m_boxInertia;
	D3DXMATRIX	m_boxInvInertia;
	D3DXMATRIX	m_invInertia;
	
	D3DXVECTOR3	m_linVelocity;
	D3DXVECTOR3 m_angVelocity;

	D3DXVECTOR3 m_linAcceleration;
	D3DXVECTOR3 m_angAcceleration;

	D3DXVECTOR3 m_forces;
	D3DXVECTOR3 m_torques;
	float		m_radius;

	Cube()
	{
		Cube( D3DXVECTOR3(0,0,0),  D3DXVECTOR3(0,0,0),		 D3DXVECTOR3(5,5,5), 1.0f);
	}

	Cube(D3DXVECTOR3& pos,
		 D3DXVECTOR3& rot,
		 D3DXVECTOR3& size,
		 float mass)
	{
		m_c = pos;
		m_e = size;
		m_e *= 0.5f;

		D3DXQUATERNION QX; D3DXQuaternionRotationAxis(&QX, &D3DXVECTOR3(1,0,0), rot.x);
		D3DXQUATERNION QY; D3DXQuaternionRotationAxis(&QY, &D3DXVECTOR3(0,1,0), rot.y);
		D3DXQUATERNION QZ; D3DXQuaternionRotationAxis(&QZ, &D3DXVECTOR3(0,0,1), rot.z);
		m_rot  = QZ;
		m_rot *= QY;
		m_rot *= QX;
		D3DXQuaternionNormalize(&m_rot, &m_rot);

		m_mass = mass;
		m_linVelocity = D3DXVECTOR3(0,0,0);
		m_angVelocity = D3DXVECTOR3(0,0,0);

		m_linAcceleration = D3DXVECTOR3(0,0,0);
		m_angAcceleration = D3DXVECTOR3(0,0,0);

		m_forces = D3DXVECTOR3(0,0,0);
		m_torques = D3DXVECTOR3(0,0,0);
		m_radius = sqrtf(size.x*size.x + size.y*size.y + size.z*size.z) + 0.1f;

		UpdateMatrix();
	}

	void AddRotateX(float xRotInc)
	{
		D3DXQUATERNION QX; D3DXQuaternionRotationAxis(&QX, &D3DXVECTOR3(1,0,0), xRotInc);
		m_rot *= QX;
		D3DXQuaternionNormalize(&m_rot, &m_rot);
		UpdateMatrix();
	}

	void AddRotateZ(float zRotInc)
	{
		D3DXQUATERNION QZ; D3DXQuaternionRotationAxis(&QZ, &D3DXVECTOR3(0,0,1), zRotInc);
		m_rot *= QZ;
		D3DXQuaternionNormalize(&m_rot, &m_rot);
		UpdateMatrix();
	}



	//------
	D3DXVECTOR3 m_cTemp;		
	D3DXVECTOR3 m_uTemp[3];		
	D3DXVECTOR3 m_eTemp;		

	D3DXMATRIX  m_matWorldTemp;
	D3DXQUATERNION m_rotTemp;


	float		m_massTemp;
	D3DXMATRIX	m_boxInertiaTemp;
	D3DXMATRIX	m_boxInvInertiaTemp;
	D3DXMATRIX	m_invInertiaTemp;
	
	D3DXVECTOR3	m_linVelocityTemp;
	D3DXVECTOR3 m_angVelocityTemp;

	D3DXVECTOR3 m_linAccelerationTemp;
	D3DXVECTOR3 m_angAccelerationTemp;

	D3DXVECTOR3 m_forcesTemp;
	D3DXVECTOR3 m_torquesTemp;

	void Store()
	{
		m_cTemp = m_c;		
		memcpy(m_uTemp, m_u, sizeof(m_u));		
		m_eTemp = m_e;		

		m_matWorldTemp = m_matWorld;
		m_rotTemp = m_rot;


		m_massTemp = m_mass;
		m_boxInertiaTemp = m_boxInertia;
		m_boxInvInertiaTemp = m_boxInvInertia;
		m_invInertiaTemp = m_invInertia;
		
		m_linVelocityTemp = m_linVelocity;
		m_angVelocityTemp = m_angVelocity;

		m_linAccelerationTemp = m_linAcceleration;
		m_angAccelerationTemp = m_angAcceleration;

		m_forcesTemp = m_forces;
		m_torquesTemp = m_torques;
	}

	void Restore()
	{
		m_c = m_cTemp;		
		memcpy(m_u, m_uTemp, sizeof(m_u));		
		m_e = m_eTemp;		

		m_matWorld = m_matWorldTemp;
		m_rot = m_rotTemp;


		m_mass = m_massTemp;
		m_boxInertia = m_boxInertiaTemp;
		m_boxInvInertia = m_boxInvInertiaTemp;
		m_invInertia = m_invInertiaTemp;
		
		m_linVelocity = m_linVelocityTemp;
		m_angVelocity = m_angVelocityTemp;

		m_linAcceleration = m_linAccelerationTemp;
		m_angAcceleration = m_angAccelerationTemp;

		m_forces = m_forcesTemp;
		m_torques = m_torquesTemp;
	}
	//------

	void UpdateMatrix()
	{
		D3DXMATRIX matR;
		D3DXMatrixRotationQuaternion(&matR, &m_rot);

		m_u[0] = D3DXVECTOR3(1,0,0);
		m_u[1] = D3DXVECTOR3(0,1,0);
		m_u[2] = D3DXVECTOR3(0,0,1);
		D3DXMATRIX matT;
		D3DXVec3TransformCoord(&m_u[0], &m_u[0], &matR);
		D3DXVec3TransformCoord(&m_u[1], &m_u[1], &matR);
		D3DXVec3TransformCoord(&m_u[2], &m_u[2], &matR);
		D3DXMatrixTranslation(&matT, m_c.x, m_c.y, m_c.z);
		m_matWorld = matR * matT;


		D3DXVECTOR3 size = m_e * 2.0f;

		float x2 = (size.x * size.x);
		float y2 = (size.y * size.y);
		float z2 = (size.z * size.z);
		float ix = (y2 + z2) * m_mass / 12.0f;
		float iy = (x2 + z2) * m_mass / 12.0f;
		float iz = (x2 + y2) * m_mass / 12.0f;
	
	
		m_boxInertia = D3DXMATRIX(	ix, 0, 0,  0,
									0, iy, 0,  0,
									0,  0, iz, 0,
									0,  0, 0,  1);


		m_invInertia = Inverse(matR) * Inverse(m_boxInertia) * matR;
	}

	void Render(IDirect3DDevice9* pDevice);

	void Update(float dt);


	void AddForce(D3DXVECTOR3& p, D3DXVECTOR3& f)
	{
		m_forces += f;
		m_torques += Cross( (p - m_c), f );
	}

	void AddCollisionImpulse(D3DXVECTOR3& p, D3DXVECTOR3& n, float dt)
	{
		D3DXVECTOR3 R = p - m_c;

		D3DXVECTOR3 V = m_linVelocity + Cross(m_angVelocity, R);

		float numer = -(1 + 0.3f) * Dot(n, V); // 0.3 Coefficient of Restitution

		float denom = 1/m_mass +  Dot(Cross((Cross(R, n) * m_invInertia),R),n);
		
		float impulse = numer / denom;

		if (impulse < 0.0f)
			return;

		float force = impulse / dt;
		
		AddForce(p, n * force);
	}

	//----------------------------------------------------------
	// 
	// j =			                           -(1+Cor)(relv.norm)
	//	     -----------------------------------------------------------------------------------
	//	     norm.norm(1/Mass0 + 1/Mass1) + (sqr(r0 x norm) / Inertia0) + (sqr(r1 x norm) / Inertia1)
	//
	//----------------------------------------------------------
	static void AddCollisionImpulse2(Cube& c0, Cube& c1, D3DXVECTOR3& p, D3DXVECTOR3& n, float dt, float Cg = 0.3f)
	{
		if (dt <= 0.001f)
		{
			return;
		}

		D3DXVECTOR3 r0 = p - c0.m_c;
		D3DXVECTOR3 r1 = p - c1.m_c;

		D3DXVECTOR3 v0 = c0.m_linVelocity + Cross(c0.m_angVelocity, r0);
		D3DXVECTOR3 v1 = c1.m_linVelocity + Cross(c1.m_angVelocity, r1);

		D3DXVECTOR3 relVel = v0 - v1;

		float numer = -(1 + Cg) * Dot(n, relVel); // 0.3 Coefficient of Restitution

		float denom = Dot(n,n)*(1/c0.m_mass + 1/c1.m_mass) +  
			
						Dot( n, 
								
								Cross((Cross(r0, n) * c0.m_invInertia), r0) +
						
								Cross((Cross(r1, n) * c1.m_invInertia), r1)
							);
		
		float impulse = numer / denom;

		if (impulse < 0.0)
			return;

		float force = impulse / dt;
		
		c0.AddForce(p,  n * force);
		c1.AddForce(p, -n * force);
	}


};


#endif //CUBE_H


